@@ -37,7 +37,6 @@ class SoundDataset(Dataset):
         self,
         folder,
         exts = ['flac', 'wav'],
-        normalize = False,
         max_length: OptionalIntOrTupleInt = None,
         target_sample_hz: OptionalIntOrTupleInt = None,
         seq_len_multiple_of: OptionalIntOrTupleInt = None
@@ -50,7 +49,6 @@ class SoundDataset(Dataset):
         assert len(files) > 0, 'no sound files found'
 
         self.files = files
-        self.normalize = normalize
 
         self.target_sample_hz = cast_tuple(target_sample_hz)
         num_outputs = len(self.target_sample_hz)
@@ -66,7 +64,7 @@ class SoundDataset(Dataset):
     def __getitem__(self, idx):
         file = self.files[idx]
 
-        data, sample_hz = torchaudio.load(file, normalize = self.normalize)
+        data, sample_hz = torchaudio.load(file)
 
         assert data.numel() > 0, f'one of your audio file ({file}) is empty. please remove it from your folder'
 
